/* Glo PID Ball Balancer Sketch
 * Designed by Om Anavekar
 * www.omlabs.me
 * 3/22/2021
*/

#include <Wire.h>
#include <Servo.h>

#include "Adafruit_VL53L0X.h"
#include <Adafruit_NeoPixel.h>

// Which pin on the Arduino is connected to the NeoPixels?
#define LED_PIN    13 //Set this to your pin

// How many NeoPixels are attached to the Arduino?
#define LED_COUNT 10

//Initialize Neopixel Object
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

//Initialize VL53L0X Object
Adafruit_VL53L0X lox = Adafruit_VL53L0X();

//Initialize Servo Object
Servo myservo;

int interval = 100; 
float distance = 0.0;
float error;
float old_error;

//-------------Variables to Change----------------

float kp=1.8; //Proportional Constant
float ki=0.2; //Integral Constant
float kd=350; //Derivative Constant

float setpoint = 45; //Middle of Lever
int deadBandMin = -5; //Minimum deadband
int deadBandMax = 5; //Maximum deadband

int servoMin = 5; //Minimum angle of servo (prevents servo from crashing into baseplate)
int servoMax = 70; //Maximum angle of servo

//-------------------------------------------------

float proportional;
float integral;
float derivative;
float PID_total;

int idx;

//Measurement function with smoothing
float measure(int sample)
{
  VL53L0X_RangingMeasurementData_t measure;
  float dist = 0.0;
  int count = 0; 
  lox.rangingTest(&measure, false); // pass in 'true' to get debug data printout!

  for (int i =0; i < sample; i++){
    // Only add measurement if it isnt an error value
    if (measure.RangeStatus != 4) {  // phase failures have incorrect data
      dist += measure.RangeMilliMeter;
      count += 1;
    } 
  }

  //Return average of measurements, taking into account bad data
  dist = dist/count;
  
  return(dist);

}

void setup() {
  
  //Begin Serial Comms
  Serial.begin(9600);  
  
  //LIDAR Error Message
  Serial.println("Adafruit VL53L0X test");
  if (!lox.begin()) {
    Serial.println(F("Failed to boot VL53L0X"));
    while(1);
  }
  
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.clear();            // Turn OFF all pixels ASAP
  strip.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)
  
  myservo.attach(15);  // attaches the servo on pin 9 to the servo object
  myservo.write(40); // Point where lever is flat
}

void loop() {
    //Collect Distance
    distance = measure(10);
    
    //PID calculations
    error = setpoint - distance;   
    proportional = kp * error;
    float difference = error - old_error;     
    derivative = kd*((difference)/interval);

    if(deadBandMin < error && error < deadBandMax){
      integral = integral + (ki * error);
    }
    else{
      integral = 0;
    }

    //Map to Servo
    PID_total = map(proportional + integral + derivative, -150, 150, 0, 150);
    
    //Write to Servo
    myservo.write(constrain(PID_total, servoMin, servoMax));  

    //Map to LED strip
    idx = map(distance, 25, 110, 0, 9);
    
    //Set LED Strip Colors
    for(int i=0;i<strip.numPixels();i++){
      strip.setPixelColor(i, 0,0,255);
    }
       
    strip.setPixelColor(idx, 238,130,238);   
    strip.show();
    
    //Set error as old
    old_error = error;
         
    delay(interval);
    
    strip.clear(); 
}

  
  
